;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: Counter8_1.asm
;;   Version: 2.2, Updated on 2003/11/10 at 16:38:18
;;  Generated by PSoC Designer ver 4.1 b931 : 21 January, 2004
;;
;;  DESCRIPTION: Counter8 User Module software implementation file
;;               for the 25/26xxx PSoC family of devices
;;
;;  NOTE: User Module APIs conform to the fastcall convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API
;;        function returns. Even though these registers may be preserved now,
;;        there is no guarantee they will be preserved in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress MicroSystems 2000-2003. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "m8c.inc"
include "Counter8_1.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------
export  Counter8_1_EnableInt
export _Counter8_1_EnableInt
export  Counter8_1_DisableInt
export _Counter8_1_DisableInt
export  Counter8_1_Start
export _Counter8_1_Start
export  Counter8_1_Stop
export _Counter8_1_Stop
export  Counter8_1_WritePeriod
export _Counter8_1_WritePeriod
export  Counter8_1_WriteCompareValue
export _Counter8_1_WriteCompareValue
export  Counter8_1_bReadCompareValue
export _Counter8_1_bReadCompareValue
export  Counter8_1_bReadCounter
export _Counter8_1_bReadCounter

; The following functions are deprecated and subject to omission in future releases
;
export  bCounter8_1_ReadCompareValue  ; deprecated
export _bCounter8_1_ReadCompareValue  ; deprecated
export  bCounter8_1_ReadCounter       ; deprecated
export _bCounter8_1_ReadCounter       ; deprecated


AREA bss (RAM,REL)

;-----------------------------------------------
;  Constant Definitions
;-----------------------------------------------

CONTROL_REG_START_BIT:         equ 0x01    ; Control register start bit
INPUT_REG_NULL:                equ 0x00    ; Clear the input register
INPUT_REG_CLOCK_MASK:          equ 0x0F    ; Input register clock mask


;-----------------------------------------------
; Variable Allocation
;-----------------------------------------------


AREA UserModules (ROM, REL)

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter8_1_EnableInt
;
;  DESCRIPTION:
;     Enables this counter's interrupt by setting the interrupt enable mask bit
;     associated with this User Module. This function has no effect until and
;     unless the global interrupts are enabled (for example by using the
;     macro M8C_EnableGInt).
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None.
;  RETURNS:      Nothing.
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Counter8_1_EnableInt:
_Counter8_1_EnableInt:
   M8C_EnableIntMask Counter8_1_INT_REG, Counter8_1_INT_MASK
   ret

.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter8_1_DisableInt
;
;  DESCRIPTION:
;     Disables this counter's interrupt by clearing the interrupt enable
;     mask bit associated with this User Module.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Counter8_1_DisableInt:
_Counter8_1_DisableInt:
   M8C_DisableIntMask Counter8_1_INT_REG, Counter8_1_INT_MASK
   ret

.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter8_1_Start
;
;  DESCRIPTION:
;     Sets the start bit in the Control register of this user module.  The
;     counter will begin counting on the next input clock as soon as the
;     enable input is asserted high.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Counter8_1_Start:
_Counter8_1_Start:
   or    reg[Counter8_1_CONTROL_REG], CONTROL_REG_START_BIT
   ret

.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter8_1_Stop
;
;  DESCRIPTION:
;     Disables counter operation by clearing the start bit in the Control
;     register.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Counter8_1_Stop:
_Counter8_1_Stop:
   and   reg[Counter8_1_CONTROL_REG], ~CONTROL_REG_START_BIT
   ret

.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter8_1_WritePeriod
;
;  DESCRIPTION:
;     Write the 8-bit period value into the Period register (DR1).
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: fastcall BYTE bPeriodValue (passed in A)
;  RETURNS:   Nothing
;  SIDE EFFECTS:
;     If the counter user module is stopped, then this value will also be
;     latched into the Count register (DR0).
;     REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Counter8_1_WritePeriod:
_Counter8_1_WritePeriod:
   tst   reg[Counter8_1_CONTROL_REG], CONTROL_REG_START_BIT
   jnz   .CounterRunning

   ; The Counter is stopped.  We have to set the clock low for
   ; the write to the Period register to cause the data to be
   ; immediately transferred into the Count register, DR0.

   mov   X, A                                    ; X <- Period
   M8C_SetBank1
   mov   A, reg[Counter8_1_INPUT_REG]            ; A <- INPUT (Clock) setting
                                                 ; Force the clock signal low
   and   reg[Counter8_1_INPUT_REG], ~INPUT_REG_CLOCK_MASK
   M8C_SetBank0
   swap  A, X                                    ; A<-Period; X<-Clock
   mov   reg[Counter8_1_PERIOD_REG], A           ; Set the new Period
   mov   A, X                                    ; A<-Clock; X<-Clock
   M8C_SetBank1
   mov   reg[Counter8_1_INPUT_REG], A            ; Restore the clock
   M8C_SetBank0
   ret

   ; The counter is running - write the period into the Period register.
   ; Upon Terminal Count this value will get loaded into the Count register.
.CounterRunning:
   mov   reg[Counter8_1_PERIOD_REG], A
   ret

.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter8_1_WriteCompareValue
;
;  DESCRIPTION:
;     Writes compare value into the Compare register (DR2).
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    fastcall BYTE bCompareValue (passed in A)
;  RETURNS:      Nothing
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Counter8_1_WriteCompareValue:
_Counter8_1_WriteCompareValue:
   mov   reg[Counter8_1_COMPARE_REG], A
   ret

.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter8_1_bReadCompareValue
;
;  DESCRIPTION:
;     Reads the Compare register.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;  RETURNS:      fastcall BYTE bCompareValue (value of DR2 in the A register)
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Counter8_1_bReadCompareValue:
_Counter8_1_bReadCompareValue:
 bCounter8_1_ReadCompareValue:                   ; this name deprecated
_bCounter8_1_ReadCompareValue:                   ; this name deprecated
   mov   A, reg[Counter8_1_COMPARE_REG]
   ret

.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: Counter8_1_bReadCounter
;
;  DESCRIPTION:
;     Returns the value in the Count register (DR0), preserving the value in
;     the compare register (DR2). Interrupts are prevented during the transfer
;     from the Count to the Compare registers by holding the clock low in
;     the PSoC block.
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: None
;  RETURNS:   fastcall BYTE bCount (value of DR0 in the A register)
;  SIDE EFFECTS:
;     1) If running, the user module is stopped momentarily and one or more
;        counts may be missed.
;     2) REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
 Counter8_1_bReadCounter:
_Counter8_1_bReadCounter:
 bCounter8_1_ReadCounter:                        ; this name deprecated
_bCounter8_1_ReadCounter:                        ; this name deprecated

   bOrigCompareValue:      EQU   0               ; Frame offset to temp Compare store
   bOrigControlReg:        EQU   1               ; Frame offset to temp CR0     store
   bOrigClockSetting:      EQU   2               ; Frame offset to temp Input   store
   wCounter:               EQU   3               ; Frame offset to temp Count   store
   STACK_FRAME_SIZE:       EQU   4               ; max stack frame size is 4 bytes

   mov   X, SP                                   ; X <- stack frame pointer
   mov   A, reg[Counter8_1_COMPARE_REG]          ; Save the Compare register on the stack
   push  A                                       ;
   mov   A, reg[Counter8_1_CONTROL_REG]          ; Save CR0 (running or stopped state)
   push  A                                       ;
                                                 ; Disable (stop) the Counter if running
   and   reg[Counter8_1_CONTROL_REG], ~CONTROL_REG_START_BIT
   M8C_SetBank1                                  ;
   mov   A, reg[Counter8_1_INPUT_REG]            ; save the clock input setting
   push  A                                       ;   on the stack (now 2 bytes) and ...
                                                 ;   hold the clock low:
   mov   reg[Counter8_1_INPUT_REG], INPUT_REG_NULL
   M8C_SetBank0
                                                 ; Extract the Count via DR2 register
   mov   A, reg[Counter8_1_COUNTER_REG]          ; DR2 <- DR0
   mov   A, reg[Counter8_1_COMPARE_REG]          ; Stash the Count on the stack
   push  A                                       ;  -stack frame is now 3 bytes
   mov   A, [X+bOrigCompareValue]                ; Restore the Compare register
   mov   reg[Counter8_1_COMPARE_REG], A
   M8C_SetBank1                                  ; Restore the counter operation:
   mov   A, [X+bOrigClockSetting]                ;   First, the clock setting...
   mov   reg[Counter8_1_INPUT_REG], A            ;
   M8C_SetBank0                                  ;   then re-enable (start) the counter
   mov   A, [X+bOrigControlReg]                  ;     if it was running when
   mov   reg[Counter8_1_CONTROL_REG], A          ;     this function was first called
   pop   A                                       ; Setup the return value
   ADD   SP, -(STACK_FRAME_SIZE-1)               ; Zap remainder of stack frame
   ret

.ENDSECTION

; End of File Counter8_1.asm
