;-----------------------------------------------------------------------------
;	This Program written for PSoC201 Board.
;
;			Ver 0.00:2005/04/03
;		M.H.E wrote on April 3.
;
; Assembly main line
;-----------------------------------------------------------------------------
;	Port1 7		:output Encoder Pluse in/Direction FG clr Strong 001
;	Port1 0-6	:LCD control 
;
;	Port2 7		:input Rx High Z 010
;		  6		:PullDown 000
;		  5		:output Tx Strong 001
;		  4		:PullDown 000
;		  3		:input Encoder Pulse Direction FG PullDown 000
;		  2		:PullDown 000
;		  1		:input Encoder Pluse in FG PullDown 000
;		  0		:PullDown 000
;
;	Port0 7		:PullDown 000
;		  6		:PullDown 000
;		  5		:PullDown 000
;		  4		:PullDown 000
;		  3		:PullDown 000
;		  2		:PullDown 000
;		  1		:PullDown 000
;		  0		:PullDown 000
;-----------------------------------------------------------------------------
;

include "m8c.inc"       ; part specific constants and macros
include "PSoCAPI.inc"   ; PSoC API definitions for all User Modules
include "DAC9_1.inc"

export _main

DELIM:				EQU	0Dh
;CONDFG status
ReceiveDelimiter:	equ	01h
ReceiveError:		equ	02h
ReceiveDelMask:		equ	0FEh
ReceiveErrMask:		equ	0FDh
;Enc
ENC_CW:				equ	08h
ENC_Pin:			equ	02h
ENC_IN:				equ	0Ah
ENC_FG_CLR:			equ	80h
ENC_FG_CLR_MSK:		equ	7Fh
;
LCD_Port:           equ	PRT1DR	;LCD & ENC I/F CLR
Line_Chr_Count:		EQU	10h
;
WZ:					equ	01h
WCY:				equ	02h
;
;Display position
DSP_FUNC_Name_Row:	equ	1
DSP_FUNC_Name_Col:	equ	0
;
DSP_FUNC_Valu_Row:	equ	1
DSP_FUNC_Valu_Col:	equ	5
;
DSP_Unit_Name_Row:	equ	1
DSP_Unit_Name_Col:	equ	0Dh
;
ZeroAdjMH:			equ	00h
ZeroAdjML:			equ	00h
ZeroAdjPH:			equ	00h
ZeroAdjPL:			equ	00h
;
;********************************************************
area 	bss(RAM)					;inform assembler that variables follow
;bit0:RS_1_ComandLineEnd
ConditionFG:		blk	1
;
PRT2BAK:			blk	1
PRT0BAK:			blk	1
;
ENC_CounterH:		blk	1		;ncoder's 16bits counter upper byte
ENC_CounterL:		blk	1		;ncoder's 16bits counter lower byte
ENC_Counter:		blk	1		;ncoder's 8bits counter
ENC_Buffer:			blk	1
;
RS_1_BuffIndex:		blk	1
RS_1_Buff:			blk	24
;
LCD_Index:			blk	1
LCD_Buff:			blk	20
LCD_row_BAK:		blk	1
LCD_col_BAK:		blk	1
;
CALwork0:			blk	1
CALwork1:			blk	1
CALwork2:			blk	1
CALwork3:			blk	1
CALwork4:			blk	1
CALwork5:			blk	1
CALwork6:			blk	1
CALwork7:			blk	1
CALREGH:			blk	1
CALREGL:			blk	1
CALREGD:			blk	1
CALREGE:			blk	1
CALREGB:			blk	1
CALREGC:			blk	1
CALFG:				blk	1
WKBUF:				blk	1
;
VAL_DAH:			blk	1
VAL_DAL:			blk	1
VAL_DAPH:			blk	1
VAL_DAPL:			blk	1
VAL_DAMH:			blk	1
VAL_DAML:			blk	1
STRwork_Index:		blk	1
STRwork:			blk	20
;
area	text(ROM,REL)				;inform assembler that program code follws

;
;****************************************************************		
_main:
	call	LCD_1_Start				; Initialize LCD  
;
	mov		A,00h					; Set cursor postion at row = 0  
	mov		X,01h					; col = 1  
	call	LCD_1_Position
;
	mov		A,>STR_Titl_01			; Load pointer to ROM string  
	mov		X,<STR_Titl_01  
	call	LCD_1_PrCString			; Print constant string  
;
	mov		A, UART_1_PARITY_NONE	;Initialize UART
	call	UART_1_Start
	call	Counter8_1_Start		;Start Baurate counter
;
	mov		[RS_1_BuffIndex],RS_1_Buff	;SET RS buff point
;
	mov		A,[Port_1_Data_SHADE]	;Preset ENC FG CLR pin
	OR		A,ENC_FG_CLR
	mov		[Port_1_Data_SHADE],A	;Save P1 data buff
    mov		reg[LCD_Port],A;		;Write P1 data
;
	mov		A, DAC9_1_LOWPOWER		; specify DAC's amplifier power  
	call	DAC9_1_Start			; and turn it on.  
;
	mov		A, DAC9_2_LOWPOWER		; specify DAC's amplifier power  
	call	DAC9_2_Start			; and turn it on.  
;
	mov		A,ZeroAdjPH				; ENC upper byte
	mov		[VAL_DAPH],A			; DA upper byte
	mov		X,A
	mov		A,ZeroAdjPL				; ENC lower byte
	mov		[VAL_DAPL],A			; DA lower byte
	call	DAC9_1_WriteStall
;
	mov		A,ZeroAdjMH				; ENC upper byte
	mov		[VAL_DAMH],A			; DA upper byte
	mov		X,A
	mov		A,ZeroAdjML				; ENC upper byte
	mov		[VAL_DAML],A			; DA lower byte
	call	DAC9_2_WriteStall
;
	mov		A,02h					; ENC upper byte
	mov		[VAL_DAH],A				; DA upper byte
	mov		A,00h					; ENC lower byte
	mov		[VAL_DAL],A				; DA lower byte
;
	call	DSP_FNC_Name_DCV
	call	DSP_Unit_Name_V
	call	DSP_FNC_Value_DCV
;
;*************************************
;	Main Loop
;*************************************
loop1:
	call	_RS_1_CommandReceive
	mov		A,[ConditionFG]
	and		A,ReceiveDelimiter
	jnz		CMDanalyzeP
	mov		A,reg[PRT2DR]
	mov		[PRT2BAK],A
	and		A,ENC_Pin
	jnz		ENCanalyzeP
	jmp		loop1
;
;***********************************
CMDanalyzeP:
	call	_CMDanalyze
	mov		[RS_1_BuffIndex],RS_1_Buff
	jmp		loop1
;
;************************************
ENCanalyzeP:
	call	ENC2DA
	call	DA_Data_setting
	call	DSP_FNC_Value_DCV
	jmp		loop1
;
;****************************************************************
;
;****************************************************************
;	Display Function Name
;****************************************************************
DSP_FNC_Name_CLR:
	mov    A,DSP_FUNC_Name_Row		; Set cursor postion at row
	mov    X,DSP_FUNC_Name_Col		; Set cursor postion at col
	call   LCD_1_Position
;
	mov		A,>STR_Func_Name_CLR		; Load pointer to ROM string  
	mov		X,<STR_Func_Name_CLR
	call	LCD_1_PrCString		; Print constant string  
;
	mov		A,DSP_FUNC_Name_Row		; Set cursor postion at row
	mov		X,DSP_FUNC_Name_Col		; Set cursor postion at col
	call	LCD_1_Position
	ret
;
;****************************************************************
DSP_FNC_Name_DCV:
	call	DSP_FNC_Name_CLR
;
	mov		A,>STR_Func_Name_DCV	; Load pointer to ROM string  
	mov		X,<STR_Func_Name_DCV
	call	LCD_1_PrCString			; Print constant string  
	ret
;
;****************************************************************
;	Display Function Value
;****************************************************************
DSP_FNC_Value_CLR:
	mov    A,DSP_FUNC_Valu_Row		; Set cursor postion at row
	mov    X,DSP_FUNC_Valu_Col		; Set cursor postion at col
	call   LCD_1_Position
;
	mov		A,>STR_Func_Valu_CLR	; Load pointer to ROM string  
	mov		X,<STR_Func_Valu_CLR
	call	LCD_1_PrCString			; Print constant string  
;
	mov		A,DSP_FUNC_Valu_Row		; Set cursor postion at row
	mov		X,DSP_FUNC_Valu_Col		; Set cursor postion at col
	call	LCD_1_Position
	ret
;
;****************************************************************
DSP_Unit_Name_V:
	mov    A,DSP_Unit_Name_Row		; Set cursor postion at row
	mov    X,DSP_Unit_Name_Col		; Set cursor postion at col
	call   LCD_1_Position
;
	mov		A,>STR_Unit_Name_V		; Load pointer to ROM string  
	mov		X,<STR_Unit_Name_V
	call	LCD_1_PrCString			; Print constant string  
	ret
;
;****************************************************************
DSP_FNC_Value_DCV:
	call	DSP_FNC_Value_CLR
;
	mov		A,[VAL_DAH]				; ENC
	and		A,02h
	jz		DSP_FNC_Value_DCVM
;
DSP_FNC_Value_DCVP:
	mov		A,[VAL_DAH]
	and		A,01h
	mov		[CALREGH],A
	mov		[CALREGL],[VAL_DAL]
	and		[CALFG],~WZ
	call	WMUL5
	and		[CALFG],~WCY
	call	LCD_Disp_16bitsDec_Sign
	jmp		DSP_FNC_Value_DCVE
;
DSP_FNC_Value_DCVM:
	mov		A,[VAL_DAH]
	cpl		A
	and		A,01h
	mov		[CALREGH],A
	mov		A,[VAL_DAL]
	cpl		A
	mov		[CALREGL],A
DSP_FNC_Value_DCVM1:
	call	WMUL5
	or		[CALFG],WCY
	call	LCD_Disp_16bitsDec_Sign
DSP_FNC_Value_DCVE:
	ret
;
;****************************************************************
;	With display function
;****************************************************************
ENC_Display_1byte:
	mov    A,[LCD_row_BAK]		; Set cursor postion at row
	mov    X,[LCD_col_BAK]		; Set cursor postion at col
	call   LCD_1_Position
;
	mov		A,[ENC_Counter]
	call	LCD_1_PrHexByte
;
	ret
;
;****************************************************************
ENC_Display_2byte:
	mov    A,[LCD_row_BAK]		; Set cursor postion at row
	mov    X,[LCD_col_BAK]		; Set cursor postion at col
	call   LCD_1_Position
;
	mov		[CALREGH],[ENC_CounterH]
	mov		[CALREGL],[ENC_CounterL]
	call	LCD_Disp_16bitsDec
;
	ret
;
;****************************************************************
;	Set DA data
;****************************************************************
;	ENC to DA
;****************************************************************
ENC2DA:
	mov		A,[VAL_DAH]				; DA upper byte to ENC upper byte
	mov		[ENC_CounterH],A
	mov		A,[VAL_DAL]				; DA lower byte to ENC lower byte
	mov		[ENC_CounterL],A
;
	call	ENC_Count_2byte
;
ENC2DAb:
	mov		A,[ENC_CounterH]		; ENC upper byte to DA upper byte
	mov		[CALREGH],A
	mov		A,[ENC_CounterL]		; ENC lower byte to DA lower byte
	mov		[CALREGL],A
	mov		[CALREGD],00h
	mov		[CALREGE],00h
	call	WCMP
	and		[CALFG],WZ
	jnz		ENC2DA1
	mov		[CALREGD],03h
	mov		[CALREGE],FFh
	call	WCMP
	and		[CALFG],WZ
	jnz		ENC2DA2
	jmp		ENC2DACOM
;
ENC2DA1:
	call	WINC
	jmp		ENC2DACOM
ENC2DA2:
	call	WDEC
	jmp		ENC2DACOM
;
ENC2DACOM:
	mov		A,[CALREGH]
	and		A,03h
	mov		[VAL_DAH],A
	mov		X,A
	mov		A,[CALREGL]
	mov		[VAL_DAL],A
;	call	DA_Data_setting
	ret
;
;****************************************************************
;
;****************************************************************
DA_Data_setting:
	mov		A,[VAL_DAH]
	and		A,02h
	jz		DA_Data_set1
;
	mov		A,[VAL_DAH]
	and		A,01h
	mov		[VAL_DAPH],A
	mov		[VAL_DAPL],[VAL_DAL]
	call	DAVP
	jmp		DA_Data_setE
;
DA_Data_set1:
	mov		A,[VAL_DAH]
	and		A,01h
	mov		[VAL_DAMH],A
	mov		[VAL_DAML],[VAL_DAL]
	call	DAVM
;
DA_Data_setE:
	ret
;
;***************************************************
DAVP:
	mov		A,[VAL_DAPH]			; Set Plus DA upper byte
	mov		X,A
	mov		A,[VAL_DAPL]			; Set Plus DA lower byte
	call	DAC9_2_WriteStall
	call	DAZM
	ret
;
;***********************************
DAVM:
	mov		A,[VAL_DAMH]			; Set Minus DA upper byte
	cpl		A
	and		A,01h
	mov		X,A
	mov		A,[VAL_DAML]			; Set Minus DA lower byte
	cpl		A
	call	DAC9_1_WriteStall
	call	DAZP
	ret
;
;***************************************************
DAZM:
	mov		A,ZeroAdjMH				; Set Minus DA Zero Data upper byte
	mov		[VAL_DAMH],A
	mov		X,A
	mov		A,ZeroAdjML				; Set Minus DA Zero Data lower byte
	mov		[VAL_DAML],A
	call	DAC9_1_WriteStall
	ret
;
;***********************************
DAZP:
	mov		A,ZeroAdjPH				; Set Plus DA Zero Data upper byte
	mov		[VAL_DAPH],A
	mov		X,A
	mov		A,ZeroAdjPL				; Set Plus DA Zero Data lower byte
	mov		[VAL_DAPL],A
	call	DAC9_2_WriteStall
	ret
;
;****************************************************************
;	Encoder count
;****************************************************************
;	1byt counter With display
;****************************************************************
ENC_Count:
	call	ENC_Count_1byte
	call	ENC_Display_1byte
	jmp		loop1
;
;****************************************************************
;	2byt counter With display
;****************************************************************
ENC_WCount:
	call	ENC_Count_2byte
	call	ENC_Display_2byte
	jmp		loop1
;
;****************************************************************
;	ENC 2 byte count
;****************************************************************
ENC_Count_2byte:
	call	ENC_chattering_Wait
	call	ENC_chattering_Wait
	call	ENC_chattering_Wait
;------
	mov		A,reg[PRT2DR]
	and		A,ENC_CW
	jnz		ENC_UP_2b
	mov		[CALREGL],[ENC_CounterL]
	mov		[CALREGH],[ENC_CounterH]
	mov		[CALREGE],01h
	mov		[CALREGD],00h
	call	WSUB
;	call	WDEC
	mov		[ENC_CounterL],[CALREGL]
	mov		[ENC_CounterH],[CALREGH]
	jmp		ENC_E_2b
ENC_UP_2b:
	mov		[CALREGL],[ENC_CounterL]
	mov		[CALREGH],[ENC_CounterH]
	mov		[CALREGE],01h
	mov		[CALREGD],00h
	call	WADD
;	call	WINC
	mov		[ENC_CounterL],[CALREGL]
	mov		[ENC_CounterH],[CALREGH]
ENC_E_2b:
	mov		A,[Port_1_Data_SHADE]
	and		A,ENC_FG_CLR_MSK
	mov		[Port_1_Data_SHADE],A
    mov		reg[LCD_Port],A;
	nop
	nop
	nop
	nop
	nop
	nop
	mov		A,[Port_1_Data_SHADE]
	or		A,ENC_FG_CLR
	mov		[Port_1_Data_SHADE],A
    mov		reg[LCD_Port],A;
	ret
;
;****************************************************************
;	ENC 1 byte count
;****************************************************************
ENC_Count_1byte:
	call	ENC_chattering_Wait
	call	ENC_chattering_Wait
	call	ENC_chattering_Wait
	call	ENC_chattering_Wait
;------
	mov		A,reg[PRT2DR]
	and		A,ENC_CW
	jnz		ENC_UP
	dec		[ENC_Counter]
	jmp		ENC_E
ENC_UP:
	inc		[ENC_Counter]
ENC_E:
	mov		A,[Port_1_Data_SHADE]
	and		A,ENC_FG_CLR_MSK
	mov		[Port_1_Data_SHADE],A
    mov		reg[LCD_Port],A;
	nop
	nop
	nop
	nop
	nop
	nop
	mov		A,[Port_1_Data_SHADE]
	or		A,ENC_FG_CLR
	mov		[Port_1_Data_SHADE],A
    mov		reg[LCD_Port],A;
	ret
;
;****************************************
;	Encoder chattering end waiting
;****************************************
ENC_chattering_Wait:
	mov		A,reg[PRT2DR]
	and		A,ENC_IN
	mov		[ENC_Buffer],A
;
	mov		A,reg[PRT2DR]
	and		A,ENC_IN
;	check 1
	cmp		A,[ENC_Buffer]
	jnz		ENC_chattering_Wait
;	check 2
	mov		A,reg[PRT2DR]
	and		A,ENC_IN
	cmp		A,[ENC_Buffer]
	jnz		ENC_chattering_Wait
;	check 3
	mov		A,reg[PRT2DR]
	and		A,ENC_IN
	cmp		A,[ENC_Buffer]
	jnz		ENC_chattering_Wait
;	check 4
	mov		A,reg[PRT2DR]
	and		A,ENC_IN
	cmp		A,[ENC_Buffer]
	jnz		ENC_chattering_Wait
	ret
;
;***************************************************
;	16bit ZSUP Display
;	[CALREGH]:up 8bit
;	[CALREGL]:low 8bit
;***************************************************
LCD_Disp_16bitsDec:
	mov		X,LCD_Buff
	call	HEX2DEC
	mov		[LCD_Buff+6],00h
;
	mov		X,LCD_Buff
	call	ZSUP
;
	mov		X,LCD_Buff
	call	_LCD_1_PrString
	ret
;
;***************************************************
;	16bit ZSUP Display
;	[CALFG],WCY:Sign WCY=0->'+', WCY=1->'-'
;	[CALREGH]:up 8bit
;	[CALREGL]:low 8bit
;***************************************************
LCD_Disp_16bitsDec_Sign:
	mov		A,[CALFG]
	and		A,WCY
	jz		LCD_Disp_16bitsDec_Sign1
	mov		[LCD_Buff],'-'
	jmp		LCD_Disp_16bitsDec_Sign2
LCD_Disp_16bitsDec_Sign1:
	mov		[LCD_Buff],'+'
LCD_Disp_16bitsDec_Sign2:
;
	mov		X,LCD_Buff+1
	call	HEX2DEC
	mov		[LCD_Buff+7],00h
	mov		[LCD_Buff+6],[LCD_Buff+5]
	mov		[LCD_Buff+5],[LCD_Buff+4]
	mov		[LCD_Buff+4],[LCD_Buff+3]
	mov		[LCD_Buff+3],'.'
;
	mov		X,LCD_Buff+1
	call	ZSUP
	mov		X,LCD_Buff
	call	_LCD_1_PrString
	ret
;
;***************************************************
;	Command analysis
;***************************************************
_CMDanalyze:
	and		[ConditionFG],ReceiveDelMask
;
	mov		[RS_1_BuffIndex],RS_1_Buff
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
;
;****************************
	cmp		A,'C'
	jz		CMD_C
	cmp		A,'P'
	jz		CMD_P
	cmp		A,'L'
	jz		CMD_L
	cmp		A,'V'
	jz		CMD_V
;
	JMP		CMD_notDEF
;
;*****************************************************
;	V system command
;*****************************************************
CMD_V:
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
;
	cmp		A,'W'
	jz		CMD_VW
	jmp		CMD_notDEF
;
;*****************************************************
CMD_VW:
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
;
	cmp		A,'E'
	jz		CMD_VWE
	jmp		CMD_notDEF
;
;*****************************************************
;	VWE command
;*****************************************************
CMD_VWE:
	mov		A,00h				; convert 0 to 9
	mov		[CALwork1],A		; save 10^4
;
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
	sub		A,30h
	mov		[CALwork2],A		; save 10^3
;
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
	sub		A,30h
	mov		[CALwork3],A		; save  10^2
;
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
	sub		A,30h				; save  10^1
	mov		[CALwork4],A
;
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
	sub		A,30h				; save  10^0
	mov		[CALwork5],A
;
	call	DEC2HEX
	mov		[ENC_CounterH],[CALREGH]
	mov		[ENC_CounterL],[CALREGL]
;
	call	ENC2DAb
	call	DA_Data_setting
	call	DSP_FNC_Value_DCV
;
	jmp		CMD_ACK
;
;*****************************************************
;	L system command
;*****************************************************
CMD_L:
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
;
	cmp		A,'O'
	jz		CMD_LO
	jmp		CMD_notDEF
;
;*****************************************************
CMD_LO:
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
;
	cmp		A,'C'
	jz		CMD_LOC
	jmp		CMD_notDEF
;
;*****************************************************
;	P system command
;*****************************************************
CMD_P:
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
;
	cmp		A,'R'
	jz		CMD_PR
	jmp		CMD_notDEF
;
;*****************************************************
CMD_PR:
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
;
	cmp		A,'N'
	jz		CMD_PRN
	jmp		CMD_notDEF
;
;*****************************************************
;	C system command
;*****************************************************
CMD_C:
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
;
	cmp		A,'L'
	jz		CMD_CL
	jmp		CMD_notDEF
;*****************************************************
CMD_CL:
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
;
	cmp		A,'R'
	jz		CMD_CLR
	jmp		CMD_notDEF
;
;*****************************************************
;	Communication command subroutine
;*****************************************************
;	command:LOCy,xx
;			 y:ASCII DEC 0-3
;			xx:ASCII DEC 00-20
;*****************************************************
CMD_LOC:
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
	and		A,03h				;LCD row data 30 to 33 only
	mov		[CALwork1],A		;save LCD row data
;
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
	cmp		A,','
	jnz		CMD_LOC_ERR
;
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
	and		A,03h				;LCD col 10^1 data 30 to 32
	mov		[CALwork2],A
;
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
	sub		A,30h				;LCD col 10^0 data 30 to 39
	mov		X,A
	mov		[CALwork3],X
;
	mov		A,[CALwork2]
	mov		X,00h
	cmp		A,0
	jz		CMD_LOC0
	cmp		A,1
	jz		CMD_LOC1
	cmp		A,2
	jz		CMD_LOC2
;
CMD_LOC3:
	mov		X,1Eh
	jmp		CMD_LOC0
CMD_LOC2:
	mov		X,14h
	jmp		CMD_LOC0
CMD_LOC1:
	mov		X,0ah
CMD_LOC0:
	mov		A,X
	add		A,[CALwork3]
	mov		X,A						; Set cursor postion at col
	mov		[LCD_col_BAK],A
	mov		A,[CALwork1]			; Set cursor postion at row
	mov		[LCD_row_BAK],A
	call   LCD_1_Position
	jmp		CMD_ACK
;
CMD_LOC_ERR:
	jmp		CMD_notDEF
;
;*****************************************************
;	command:CL
;	LCD CLR
;*****************************************************
CMD_CLR:
	inc		[RS_1_BuffIndex]
	mov		A,00h					; Set cursor postion at row = 0  
	mov		X,00h					; col = 0
	call   LCD_1_Position
	mov		A,Line_Chr_Count		;
	call	STRspc_CLR
;
	mov		A,01h					; Set cursor postion at row = 0
	mov		X,00h					; col = 0
	call   LCD_1_Position
	mov		A,Line_Chr_Count		;
	call	STRspc_CLR
;
	jmp		CMD_ACK
;
;*****************************************************
;	command:PRNxxxxxxx
;	LCD Display xxxxxxx
;*****************************************************
CMD_PRN:
	mov		[LCD_Index],LCD_Buff
CMD_PRN_Loop:
	inc		[RS_1_BuffIndex]
	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
	mov		X,[LCD_Index]
	mov		[X],A
	inc		[LCD_Index]
	cmp		A,00h
	jz		CMD_PRN_E1
	jmp		CMD_PRN_Loop
;
CMD_PRN_E1:
	mov		X,LCD_Buff
	call	_LCD_1_PrString
;
	jmp	CMD_ACK
;
;***********************************************
;	Response command
;***********************************************
;	Invalid command reception
;***********************************************
CMD_notDEF:
	mov		A,'?'
    call	Tx1Chr					; transmit the response data
	mov		A,DELIM
    call	Tx1Chr					; transmit the response data
	ret
;
;************************************************
;	Effective command reception
;***********************************************
CMD_ACK:
	mov		A,'*'
    call	Tx1Chr					; transmit the response data
	mov		A,DELIM
    call	Tx1Chr					; transmit the response data
	ret
;
;*******************************************************
;	One character transmission
;*******************************************************
Tx1Chr:
	tst	  reg[UART_1_TX_CONTROL_REG],UART_1_TX_BUFFER_EMPTY
	jz	  Tx1Chr
    call	UART_1_SendData			; transmit the response data
	ret
;
;*******************************************************
;	One line transmission
;*******************************************************
.TxData:
	mov		[RS_1_BuffIndex],RS_1_Buff
TxLoop:
	tst	  reg[UART_1_TX_CONTROL_REG],UART_1_TX_BUFFER_EMPTY
	jz	  TxLoop

	mov		X,[RS_1_BuffIndex]
	mov		A,[X]
	jz		TxLoopEnd
    call	UART_1_SendData			; transmit the response data   
	inc		[RS_1_BuffIndex]
	jmp		TxLoop
TxLoopEnd:
	mov		A,DELIM
    call	UART_1_SendData			; transmit the response data
	mov		[RS_1_BuffIndex],RS_1_Buff
    ret
;
;*******************************************************
;	One line reception
;*******************************************************
_RS_1_CommandReceive:
    call  UART_1_bReadRxStatus
    and   A, UART_RX_COMPLETE
    jz    .GetDataEnd
;
    and   A, UART_1_RX_ERROR		; data received - see if data is valid    
    jz    .GetData
	or		[ConditionFG],ReceiveError
	mov		X,[RS_1_BuffIndex]
    jmp		.GetEnd
.GetData:
    call	UART_1_bReadRxData		; read the data from the receiver
	mov		X,[RS_1_BuffIndex]
	mov		[X],A
	inc		[RS_1_BuffIndex]
	cmp		A,DELIM
	jnz		.GetDataEnd
.GetEnd:
	mov		A,00h
	mov		[X],A
	mov		[RS_1_BuffIndex],RS_1_Buff
	or		[ConditionFG],ReceiveDelimiter
.GetDataEnd:
	ret
;
;**************************************************
;	Display at blank
;		A:blank Count
;**************************************************
STRspc_CLR:
	and		A,1Fh			;Limit 31 Chr
	jz		STRspc_END
	push	A
	mov		[LCD_Index],LCD_Buff
STRspc_LP1:
	pop		A
	jz		STRspc_DSP
	DEC		A
	push	A
	mov		X,[LCD_Index]
	mov		A,20h			;SP data
	mov		[X],A
	inc		[LCD_Index]
	jmp		STRspc_LP1
STRspc_DSP:
	mov		X,[LCD_Index]
	mov		A,00h			;null data
	mov		[X],A
	mov		X,LCD_Buff
	call	_LCD_1_PrString
STRspc_END:
	ret
;
;***********************************************
;	zero suppress
;***********************************************
ZSUP:
	mov		A,[X]
	cmp		A,'0'
	jnz		ZSUPE
	mov		A,[X+1]
	cmp		A,'.'
	jz		ZSUPE
	cmp		A,00h
	jz		ZSUPE
	mov		[X],20h
	inc		X
	jmp		ZSUP
;
ZSUPE:
	ret
;
;************************************************************
;DEC2HEX:
;	input :CALwork1,CALwork5
;	output:CALREGH,CALREGL
;	Work:CALwork6,CALwork7,CALREGD,CALREGE,CALREGB,CALREGC
;
;	CALwork1: 27 10H	[X+1]
;	CALwork2: 03 E8H	[X+2]
;	CALwork3: 00 64H	[X+3]
;	CALwork4: 00 0AH	[X+4]
;	CALwork5: 00 01H	[X+5]
;************************************************************
DEC2HEX:
	mov		[CALREGH],27h
	mov		[CALREGL],10h
	mov		A,[CALwork1]
	call	WMULn
	mov		[CALwork6],[CALREGH]
	mov		[CALwork7],[CALREGL]
;
	mov		[CALREGH],03h
	mov		[CALREGL],0E8h
	mov		A,[CALwork2]
	call	WMULn
	mov		A,[CALREGH]
	add		[CALwork6],A
	mov		A,[CALREGL]
	add		[CALwork7],A
;
	mov		[CALREGH],00h
	mov		[CALREGL],064h
	mov		A,[CALwork3]
	call	WMULn
	mov		A,[CALREGH]
	add		[CALwork6],A
	mov		A,[CALREGL]
	add		[CALwork7],A
;
	mov		[CALREGH],00h
	mov		[CALREGL],0Ah
	mov		A,[CALwork4]
	call	WMULn
	mov		A,[CALREGH]
	add		[CALwork6],A
	mov		A,[CALREGL]
	add		[CALwork7],A
;
	mov		[CALREGH],00h
	mov		[CALREGL],[CALwork5]
	mov		[CALREGD],[CALwork6]
	mov		[CALREGE],[CALwork7]
	call	WADD
;
	ret
;
;************************************************************
;HEX2DEC:
;	input :CALREGH,CALREGL 00,00 00,01 00,02 .... FF,FF
;	output:X + 1 .... X + 5
;	Work:CALREGD,CALREGE
;
;	10000: 27 10H	[X+1]
;	 1000: 03 E8H	[X+2]
;	  100: 00 64H	[X+3]
;	   10: 00 0AH	[X+4]
;	    1: 00 01H	[X+5]
;************************************************************
HEX2DEC:
	mov		[CALREGD],27h
	mov		[CALREGE],10h
	mov		[X],00h
	call	H2DLoop
;
	inc		X
	mov		[CALREGD],03h
	mov		[CALREGE],0E8h
	mov		[X],00h
	call	H2DLoop
;
	inc		X
	mov		[CALREGD],00h
	mov		[CALREGE],64h
	mov		[X],00h
	call	H2DLoop
;
	inc		X
	mov		[CALREGD],00h
	mov		[CALREGE],0Ah
	mov		[X],00h
	call	H2DLoop
H2D5:
	inc		X
	mov		A,[CALREGL]
	mov		[X],A
	or		[X],30h
	ret
;
;**********************************
H2DLoop:
	mov		A,[CALREGH]
	mov		[CALREGB],A
	mov		A,[CALREGL]
	mov		[CALREGC],A
	call	WSUB
	inc		[X]				;count up
	mov		A,[CALFG]
	and		A,WCY
	jz		H2DLoop
	dec		[X]				;count down
	or		[X],30h
	mov		A,[CALREGB]
	mov		[CALREGH],A
	mov		A,[CALREGC]
	mov		[CALREGL],A
	ret
;
;***********************************************
;	HL <- HL - DE   WCY :Carry FG WZ :Zero FG
;***********************************************
WSUB:
	and		[CALFG],~WCY
	and		[CALFG],~WZ
	mov		A,[CALREGL]
	sub		A,[CALREGE]
	jnc		WSUB1
	dec		[CALREGH]
	jnc		WSUB1
	or		[CALFG],WCY		;Carry FG ON
WSUB1:
	mov		[CALREGL],A
	mov		A,[CALREGH]
	sub		A,[CALREGD]
	jnc		WSUB2
	or		[CALFG],WCY
WSUB2:
	mov		[CALREGH],A
;
	mov		A,[CALREGL]		;zero check
	or		A,[CALREGH]
	jnz		WSUBE
	or		[CALFG],WZ
WSUBE:
	ret
;
;***********************************************
;	HL CMP DE   WCY :Carry FG, WZ :Zero FG
;***********************************************
WCMP:
	and		[CALFG],~WCY
	and		[CALFG],~WZ
	mov		A,[CALREGH]
	cmp		A,[CALREGD]
	jnz		WCMPC
	mov		A,[CALREGL]
	cmp		A,[CALREGE]
	jnz		WCMPC
	or		[CALFG],WZ
	jmp		WCMPE
WCMPC:
	jnc		WCMPE
	or		[CALFG],WCY
WCMPE:
	ret
;
;***********************************************
;	HL <- HL + DE   WCY :Carry FG, WZ :Zero FG
;***********************************************
WADD:
	and		[CALFG],~WCY
	and		[CALFG],~WZ
;
	mov		A,[CALREGL]
	add		A,[CALREGE]
	jnc		WADD1
	inc		[CALREGH]
	jnc		WADD1
	or		[CALFG],WCY		;Carry FG ON
WADD1:
	mov		[CALREGL],A
	mov		A,[CALREGH]
	ADD		A,[CALREGD]
	jnc		WADD2
	or		[CALFG],WCY
WADD2:
	mov		[CALREGH],A
;
	mov		A,[CALREGL]		;zero check
	or		A,[CALREGH]
	jnz		WADDE
	or		[CALFG],WZ
WADDE:
	ret
;
;***********************************************
;	HL <- HL + 1   WCY :Carry FG, WZ :Zero FG
;***********************************************
WINC:
	and		[CALFG],~WCY
	and		[CALFG],~WZ
;
	inc		[CALREGL]
	jnc		WINC1
	inc		[CALREGH]
	jnc		WINC1
	or		[CALFG],WCY		;Carry FG ON
WINC1:
	mov		A,[CALREGL]		;zero check
	or		A,[CALREGH]
	jnz		WINCE
	or		[CALFG],WZ
WINCE:
	ret
;
;***********************************************
;	HL <- HL - 1   WCY :Carry FG, WZ :Zero FG
;***********************************************
WDEC:
	and		[CALFG],~WCY
	and		[CALFG],~WZ
;
	dec		[CALREGL]
	jnc		WDEC1
	dec		[CALREGH]
	jnc		WDEC1
	or		[CALFG],WCY		;Carry FG ON
WDEC1:
	mov		A,[CALREGL]		;zero check
	or		A,[CALREGH]
	jnz		WDECE
	or		[CALFG],WZ
WDECE:
	ret
;
;***********************************************
;	Multiplication
;	input:[CALREGH][CALREGL]
;	output:[CALREGH][CALREGL]
;***********************************************
WMULn:
	cmp	A,0
	jz		WMUL0
	cmp	A,1
	jz		WMUL1
	cmp	A,2
	jz		WMUL2
	cmp	A,3
	jz		WMUL3
	cmp	A,4
	jz		WMUL4
	cmp	A,5
	jz		WMUL5
	cmp	A,6
	jz		WMUL6
	cmp	A,7
	jz		WMUL7
	cmp	A,8
	jz		WMUL8
	cmp	A,9
	jz		WMUL9
	cmp	A,10
	jz		WMUL10
;
WMUL1:
	ret
WMUL0:
	mov		[CALREGH],0
	mov		[CALREGL],0
	ret
;
;****************************
WMUL2:
	asl		[CALREGL]		;Arithmetic Shift Left
	rlc		[CALREGH]		;Rotate Left through Carry
	ret
;
;****************************
WMUL3:
	mov		[CALREGE],[CALREGL]		;
	mov		[CALREGD],[CALREGH]		;
	call	WMUL2
	call	WADD
	ret
;
;****************************
WMUL4:
	asl		[CALREGL]		;Arithmetic Shift Left
	rlc		[CALREGH]		;Rotate Left through Carry
;
	asl		[CALREGL]		;Arithmetic Shift Left
	rlc		[CALREGH]		;Rotate Left through Carry
	ret
;
;****************************
WMUL5:
	mov		[CALREGE],[CALREGL]		;
	mov		[CALREGD],[CALREGH]		;
	call	WMUL4
	call	WADD
	ret
;
;****************************
WMUL6:
	call	WMUL2
	call	WMUL3
	ret
;
;****************************
WMUL7:
	mov		[CALREGC],[CALREGL]		;
	mov		[CALREGB],[CALREGH]		;
	call	WMUL2
	call	WMUL3
	mov		[CALREGE],[CALREGC]		;
	mov		[CALREGD],[CALREGB]		;
	call	WADD
	ret
;
;****************************
WMUL8:
	asl		[CALREGL]		;Arithmetic Shift Left
	rlc		[CALREGH]		;Rotate Left through Carry
;
	asl		[CALREGL]		;Arithmetic Shift Left
	rlc		[CALREGH]		;Rotate Left through Carry
;
	asl		[CALREGL]		;Arithmetic Shift Left
	rlc		[CALREGH]		;Rotate Left through Carry
	ret
;
;****************************
WMUL9:
	call	WMUL3
	call	WMUL3
	ret
;
;****************************
WMUL10:
	call	WMUL2
	call	WMUL5
	ret
;
;****************************
WMUL11:
	mov		[CALREGC],[CALREGL]		;
	mov		[CALREGB],[CALREGH]		;
	call	WMUL2
	call	WMUL5
	mov		[CALREGE],[CALREGC]		;
	mov		[CALREGD],[CALREGB]		;
	call	WADD
	ret
;
;****************************
WMUL12:
	call	WMUL4
	call	WMUL3
	ret
;
;****************************
WMUL13:
	mov		[CALREGC],[CALREGL]		;
	mov		[CALREGB],[CALREGH]		;
	call	WMUL4
	call	WMUL3
	mov		[CALREGE],[CALREGC]		;
	mov		[CALREGD],[CALREGB]		;
	call	WADD
	ret
;
;****************************
WMUL14:
	call	WMUL2
	call	WMUL7
	ret
;
;****************************
WMUL15:
	call	WMUL3
	call	WMUL5
	ret
;
;****************************
WMUL16:
	asl		[CALREGL]		;Arithmetic Shift Left
	rlc		[CALREGH]		;Rotate Left through Carry
;
	asl		[CALREGL]		;Arithmetic Shift Left
	rlc		[CALREGH]		;Rotate Left through Carry
;
	asl		[CALREGL]		;Arithmetic Shift Left
	rlc		[CALREGH]		;Rotate Left through Carry
;
	asl		[CALREGL]		;Arithmetic Shift Left
	rlc		[CALREGH]		;Rotate Left through Carry
	ret
;
;***********************************************
.LITERAL
;***********************************************
;	Translation table Hex
;***********************************************
CNV_HEX_STR::
	DS    "0123456789ABCDEF"
;
;***********************************************
;	LCD Display Data
;***********************************************
STR_Titl_01:
	DS "Sample 02"  
	DB  00h                   ; String should always be null terminated

STR_VER:
	DS "Ver. 1.00"
	DB  00h                   ; String should always be null terminated
;
;***********************************************
STR_Func_Valu_CLR:
	DS "      "
	DB  00h                   ; String should always be null terminated
STR_Func_Name_CLR:
	DS "     "  
	DB  00h                   ; String should always be null terminated
STR_Func_Name_DCV:
	DS "DC V:"  
	DB  00h                   ; String should always be null terminated
STR_Unit_Name_V:
	DS "[V]"  
	DB  00h                   ; String should always be null terminated
;
.ENDLITERAL